<?php

namespace FinSearchAPI\Components\XmlParser;


use ReflectionClass;
use ReflectionProperty;
use SimpleXMLElement;

/**
 * Class AbstractElement
 * @package Shopware\Plugins\Local\Backend\Kumavision\Parser\Model
 *
 * Base class that helps in cloning XML structure into equivalent tree structure of custom classes
 * Eliminates the problem of missing key if parent element has only one child
 *
 * @example
 *
 * <?xml version="1.0" encoding="UTF-8"?>
 * <category>
 *    <name>Category 1</name>
 *    <products>
 *        <product>
 *            <name>Product 1</name>
 *            <price>1000</price>
 *        </product>
 *        <product>
 *             <name>Product 2</name>
 *            <price>2000</price>
 *        </product>
 *    </products>
 * </category>
 *
 * Your Category class would need to have this structure:
 *
 * class Category extends AbstractElement
 * {
 *     public $name;
 *     public $products = [];
 *
 *     protected function generateChildren()
 *     {
 *         return [
 *             'products'=>function(SimpleXMLElement $child) {
 *                 $this->products[] = new Product($child);
 *              }
 *         ];
 *      }
 * }
 *
 * This is all the code required for parser.
 *
 * Note that custom __construct(SimpleXMLElement $element) method must call parent::__construct($element)
 * or the program will not work
 *
 * Accessing undefined properties will throw an Exception.
 */
abstract class AbstractElement
{
    /**
     * @var AttributeCollection
     */
    public $attributes;

    /**
     * @var SimpleXMLElement
     */
    protected $element;

    public function __construct(SimpleXMLElement $element)
    {
        $this->element = $element;
        $this->attributes = new AttributeCollection($element);
        $this->parse($element);
    }

    // Do not allow reading from undeclared properties. Allows easier debugging in future upgrades.
    public function __get($name)
    {
        throw new \Exception("Unknown property $name");
    }

    // Shopware sets many custom error handlers and suppressors. This is a way to read backtrace stack which would otherwise be inaccessible
    public function __set($name, $val)
    {
        $e = new \Exception("Unknown property $name");
        echo "Illegal write attempt at " . get_class($this) . '::$' . $name . "<br><br>";
        echo nl2br($e->getTraceAsString());
        die;
    }

    public function __debugInfo()
    {
        $result = [];
        $reflect = new ReflectionClass($this);
        $publicProperties = $reflect->getProperties(ReflectionProperty::IS_PUBLIC);

        foreach ($publicProperties as $reflectionPropertyObject) {
            $name = $reflectionPropertyObject->name;

            $fieldValue = $this->$name;

            if ($fieldValue instanceof AbstractElement) {
                $result[$name] = $fieldValue->__debugInfo();
            } elseif (is_array($fieldValue)) {
                $result[$name] = array_map(function(AbstractElement $object) {
                    return $object->__debugInfo();
                }, $fieldValue);
            } elseif ($fieldValue instanceof AttributeCollection) {
                if (count($fieldValue) > 0) {
                    $result['attributes'] = $fieldValue->toArray();
                }
            } elseif ($fieldValue instanceof Field) {
                $result[$name] = $fieldValue->__debugInfo();
            } else {
                $result[$name] = (string)$fieldValue;
            }
        }

        return $result;
    }

    protected function parse(SimpleXMLElement $element)
    {
        $generators = $this->generateChildren();
        /** @var SimpleXMLElement $child */
        foreach ($element->children() as $child) {
            $name = $child->getName();
            if (isset($generators[$name])) {
                $callback = $generators[$name];
                $callback($child);
            } else {
                $this->mapField($child);
            }
        }
    }

    protected function mapField(SimpleXMLElement $element)
    {
        $name = $element->getName();
        $this->$name = new Field($element);
    }

    /**
     * Generates children array.
     * 
     * @return array Array of child objects
     */
    protected function generateChildren()
    {
        return array();
    }
}
