<?php
/** Class ExportHelper */

/**
 * Class ExportHelper contains helper functions for FINDOLOGIC plugin export
 *
 * @author FINDOLOGIC GmbH
 * @copyright  Copyright (c) 2016
 *
 */

namespace FinSearchAPI\Helpers;

use Shopware\Bundle\SearchBundle\Condition\CategoryCondition;
use Shopware\Bundle\SearchBundle\Condition\CustomerGroupCondition;
use Shopware\Bundle\SearchBundle\Criteria;
use Shopware\Bundle\StoreFrontBundle\Service\Core\ContextService;
use Shopware\Bundle\StoreFrontBundle\Struct\ProductContext;
use Shopware\Components\ProductStream\Repository;
use Shopware\Models\Article\Article;


class ExportHelper
{
    /**
     * Get record from article table
     *
     * @param int $article_id Article ID
     * @return array Article
     */
    public function getMainArticle($article_id)
    {
        $builder = Shopware()->Models()->createQueryBuilder();
        $builder->select('article')
            ->from('Shopware\Models\Article\Article', 'article')
            ->where('article.id = ?1')
            ->andWhere('article.active = 1');

        $builder = $builder->setParameter(1, (int) $article_id);
        $query = $builder->getQuery();
        $result = $query->getOneOrNullResult(\Doctrine\ORM\AbstractQuery::HYDRATE_ARRAY);

        return $result;
    }

    /**
     * Get article details
     *
     * @param int $article_id Article ID
     * @return array Article details
     */
    public function getArticleDetails($article_id)
    {
        $builder = Shopware()->Models()->createQueryBuilder();
        $builder->select('details')
            ->from('Shopware\Models\Article\Detail', 'details')
            ->where('details.articleId = ?1')
            ->setParameter(1, (int) $article_id);

        $query = $builder->getQuery();
        $result = $query->getResult(\Doctrine\ORM\AbstractQuery::HYDRATE_ARRAY);

        return $result;
    }

    /**
     * Get media from s_filter_values table
     *
     * @param $value filter value
     * @return array
     */
    public function getMedia($value)
    {
        $builder = Shopware()->Models()->createQueryBuilder();
        $builder->select(array('s_filter_values', 'media'))
            ->from('Shopware\Models\Property\Value', 's_filter_values')
            ->leftJoin('s_filter_values.media', 'media')
            ->where('s_filter_values.value = ?1')
            ->setParameter(1, $value);

        $query = $builder->getQuery();
        $result = $query->getResult(\Doctrine\ORM\AbstractQuery::HYDRATE_ARRAY);

        return $result;
    }

    /**
     * Get article tax
     *
     * @param int $article_tax_id Article tax ID
     * @return array Article tax
     */
    public function getArticleTax($article_tax_id)
    {
        $builder = Shopware()->Models()->createQueryBuilder();
        $builder->select('tax')
            ->from('Shopware\Models\Tax\Tax', 'tax')
            ->where('tax.id = ?1')
            ->setParameter(1, (int) $article_tax_id);

        $query = $builder->getQuery();
        $result = $query->getResult(\Doctrine\ORM\AbstractQuery::HYDRATE_ARRAY);

        return $result;
    }

    /**
     * Get article images
     *
     * @param int $article_id Article ID
     * @return array Article images
     */
    public function getArticleImages($article_id)
    {
        $builder = Shopware()->Models()->createQueryBuilder();
        $builder->select('image')
            ->from('Shopware\Models\Article\Image', 'image')
            ->where('image.articleId = ?1')
            ->setParameter(1, (int) $article_id);

        $query = $builder->getQuery();
        $result = $query->getResult(\Doctrine\ORM\AbstractQuery::HYDRATE_ARRAY);

        return $result;
    }

    /**
     * Get prices for all article variants
     *
     * @param array $product Article object
     * @return void
     */
    public function getArticleDetailsPrices(&$product)
    {
        if (!empty($product) && !empty($product['details'])) {

            $builder = Shopware()->Models()->createQueryBuilder();
            $builder->select('price')
                ->from('Shopware\Models\Article\Price', 'price')
                ->where('price.articleId = ?1')
                ->setParameter(1, (int) $product['id']);
            $query = $builder->getQuery();
            $result = $query->getResult(\Doctrine\ORM\AbstractQuery::HYDRATE_ARRAY);

            // $detail['prices'][] = $result[0];

            $cnt = 0;
            foreach ($product['details'] as &$detail) {
                $detail['prices'][] = $result[$cnt];
                $cnt++;
            }
        }
    }

    /**
     * Get configurator options for article variants
     *
     * @param array $product Article object
     * @return void
     */
    public function getArticleDetailsConfiguratiorOptions(&$product)
    {
        if (!empty($product) && !empty($product['details'])) {
            foreach ($product['details'] as &$detail) {

                $builder = Shopware()->Models()->createQueryBuilder();
                $builder->select(array('detail', 'options'))
                    ->from('Shopware\Models\Article\Detail', 'detail')
                    ->leftJoin('detail.configuratorOptions', 'options')
                    ->where('detail.id = ?1')
                    ->setParameter(1, (int) $detail['id']);
                $query = $builder->getQuery();
                $result = $query->getResult(\Doctrine\ORM\AbstractQuery::HYDRATE_ARRAY);

                $detail['configuratorOptions'] = $result[0]['configuratorOptions'];
            }
        }
    }

    /**
     * Get article property values
     *
     * @param int $article_id Article ID
     * @return mixed
     */
    public function getArticlePropertyValues($article_id)
    {
        $builder = Shopware()->Models()->createQueryBuilder();
        $builder->select(array('article', 'propertyValues'))
            ->from('Shopware\Models\Article\Article', 'article')
            ->leftJoin('article.propertyValues', 'propertyValues')
            ->where('article.id = ?1')
            ->setParameter(1, (int) $article_id);

        $query = $builder->getQuery();
        $result = $query->getResult(\Doctrine\ORM\AbstractQuery::HYDRATE_ARRAY);

        return $result[0]['propertyValues'];
    }

    /**
     * Get article categories
     *
     * @param int $article_id Article ID
     * @return array Article categories
     */
    public function getArticleCategories($article_id)
    {
        $builder = Shopware()->Models()->createQueryBuilder();
        $builder->select(array('article', 'categories'))
            ->from('Shopware\Models\Article\Article', 'article')
            ->leftJoin('article.categories', 'categories')
            ->where('article.id = ?1')
            ->setParameter(1, (int) $article_id);
        $query = $builder->getQuery();
        $result = $query->getResult(\Doctrine\ORM\AbstractQuery::HYDRATE_ARRAY);

        $final_result = array();

        foreach ($result as $res) {
            $final_result[$res['categories'][0]['id']] = $res['categories'][0];
        }

        return $final_result;
    }

    /**
     * Get article product stream
     *
     * @param $article_id Article ID
     * @return array
     */
    public function getArticleProductStream($article_id)
    {
        $productStreams = "SELECT cat.*
                FROM s_product_streams AS ps
                LEFT JOIN s_product_streams_selection AS pss
                ON ps.id = pss.stream_id
                LEFT JOIN s_categories AS cat
                ON cat.stream_id = ps.id
                LEFT JOIN fin_search_api_product_streams AS flsaps
                ON ps.id = flsaps.stream_id
                WHERE pss.article_id = :article_id OR flsaps.article_id = :article_id";

        $productStreamsData = Shopware()->Db()->fetchAll($productStreams, array('article_id' => $article_id));

        // Clear enty records from dataset
        foreach ($productStreamsData as $key => $data) {
            if (empty($data['id'])) {
                unset($productStreamsData[$key]);
            }
        }

        return $productStreamsData;
    }

    /**
     * Imports data about conditional streams to 'fin_search_api_product_streams' table
     *
     * @throws \Zend_Db_Adapter_Exception
     * @return void
     */
    public function importProductStreamsDataToDb()
    {
        $productStreamsSql = "SELECT * FROM s_product_streams";
        $streams = Shopware()->Db()->fetchAll($productStreamsSql);

        $importData = [];
        foreach ($streams as $stream) {
            if (!empty($stream['conditions'])) {
                $productsTotalCount = $this->getProductStreamArticlesByConditionsTotalCount($stream['conditions'], 0, 1);
                $offset = 0;
                $limit = 200;
                $products = [];

                while ($offset < $productsTotalCount) {
                    $products = array_merge($products, $this->getProductStreamArticlesByConditions($stream['conditions'], $offset, $limit));
                    $offset += $limit;
                }

                $importData[$stream['id']] = $products;
            }
        }

        $this->truncateProductStreamsDataTable();

        foreach($importData as $streamId => $productIds) {
            foreach ($productIds as $productId) {
                Shopware()->Db()->exec("
                    INSERT INTO fin_search_api_product_streams (stream_id, article_id)
                    VALUES ({$streamId}, {$productId})
                ");
            }
        }
    }

    /**
     * Truncates 'fin_search_api_product_streams' table
     *
     * @return int
     * @throws \Zend_Db_Adapter_Exception
     */
    public function truncateProductStreamsDataTable()
    {
        return Shopware()->Db()->exec('TRUNCATE fin_search_api_product_streams');
    }

    /**
     * Checks if 'fin_search_api_product_streams' table is empty
     * Empty => true
     * Not Empty => false
     * @return bool
     * @throws \Zend_Db_Adapter_Exception
     */
    public function checkIfProductStreamsTableIsEmpty()
    {
        return !(bool)Shopware()->Db()->fetchOne('SELECT count(*) FROM fin_search_api_product_streams');
    }

    /**
     * Gets articles that belong to conditional product stream
     *
     * @param $conditions
     * @param $offset
     * @param $limit
     * @return array
     */
    protected function getProductStreamArticlesByConditions($conditions, $offset, $limit)
    {
        $result = $this->getProductStreamResults($conditions, $offset, $limit);

        $products = array_values($result->getProducts());

        /**
         * @var $product Article
         */
        $productsIds = [];
        foreach ($products as $product) {
            $productsIds[] = $product->getId();
        }

        return $productsIds;
    }

    /**
     * Gets product stream articles total count
     *
     * @param $conditions
     * @param $offset
     * @param $limit
     * @return int
     */
    protected function getProductStreamArticlesByConditionsTotalCount($conditions, $offset, $limit)
    {
        $result = $this->getProductStreamResults($conditions, $offset, $limit);

        return $result->getTotalCount();
    }

    /**
     * Gets product stream results
     *
     * @param $conditions
     * @param $offset
     * @param $limit
     * @return \Shopware\Bundle\SearchBundle\ProductSearchResult
     * @throws \Exception
     */
    private function getProductStreamResults($conditions, $offset, $limit)
    {
        $conditions = json_decode($conditions, true);

        /**
         * @var $criteria Criteria
         */
        $criteria = new Criteria();
        $criteria->offset($offset);
        $criteria->limit($limit);

        $repository = new Repository(null);
        $conditions = $repository->unserialize($conditions);

        /**
         * @var $condition \Shopware\Bundle\SearchBundle\ConditionInterface
         */
        foreach ($conditions as $condition) {
            $criteria->addCondition($condition);
        }

        $context = $this->createContext(Shopware()->Shop()->getId());

        $criteria->addBaseCondition(
            new CustomerGroupCondition([$context->getCurrentCustomerGroup()->getId()])
        );

        $category = $context->getShop()->getCategory()->getId();
        $criteria->addBaseCondition(
            new CategoryCondition([$category])
        );


        $result = Shopware()->Container()->get('shopware_search.product_search')
            ->search($criteria, $context);

        return $result;
    }

    /**
     * @param $shopId
     * @param int $currencyId
     * @param int $customerGroupKey
     * @return ProductContext
     */
    private function createContext($shopId, $currencyId = null, $customerGroupKey = null)
    {
        /** @var \Shopware\Models\Shop\Repository $repo */
        $repo = Shopware()->Container()->get('models')->getRepository('Shopware\Models\Shop\Shop');

        $shop = $repo->getActiveById($shopId);
        $shopId = $shop->getId();

        if (!$currencyId) {
            $currencyId = $shop->getCurrency()->getId();
        }

        if (!$customerGroupKey) {
            $customerGroupKey = ContextService::FALLBACK_CUSTOMER_GROUP;
        }

        return Shopware()->Container()->get('shopware_storefront.context_service')
            ->createProductContext($shopId, $currencyId, $customerGroupKey);
    }

    /**
     * Get config value 'hideNoInstock'
     *
     * @return boolean
     */
    public function getHideNoInstock()
    {
        $builder = Shopware()->Models()->createQueryBuilder();
        $builder->select(array('value', 'element'))
            ->from('\Shopware\Models\Config\Value', 'value')
            ->leftJoin('value.element', 'element')
            ->where('element.name = ?1')
            ->setParameter(1, 'hideNoInstock');
        $query = $builder->getQuery();
        $result = $query->getResult(\Doctrine\ORM\AbstractQuery::HYDRATE_ARRAY);

        return $result;
    }

    /**
     * Debugging variables
     * @param mixed $var Message to debug
     */
    private function __debug($var) {
        echo "<pre>";
        print_r($var);
        echo "</pre>";
        echo "<hr />";
    }

    /**
     * Get class and all methods for variable
     *
     * @param object $var Object to be debugged
     * @return void
     */
    private function __getMethods($var)
    {
        $class = get_class($var);
        echo $class;
        echo "<hr />";
        echo "<pre>";
        echo var_dump(get_class_methods($class));
        echo "</pre>";
        echo "<hr />";
    }
}